% Runs a mock-up ROPA problem, where the model and the system are equal and the disturbance setup is pre-determined. 
% The system setup is based on the paper:
%   Implementation of Steady-state Real-time Optimization Using Transient Measurements on Experimental Rig
%   by Matias et al.

% Other m-files required: SmoothPlantParameters2.mat; Disturbances2.mat; RigNoise.mat; InitialState_2021-03-03_104459_NoRTO_test2_1.mat; EKFconf.mat
% MAT-files required: 
%       For ROPA:
%           2. RExtendedKalmanFilter.m
%           3. ErosionRigSSOptimization.m

%       For Mockup Labview Interface:
%           1. InitializationLabViewMain.m
%           2. LabViewMain.m

%       For "Plant":
%           1. ErosionRigDynModel.m

%       Bounds, Parameters and Initial conditon:
%           1. InitialConditionGasLift.m
%           2. OptimizationBoundsGasLiftRiser2.m
%           3. ParametersGasLiftModel.m
clear
%close all
clc

%saving data
name = 'ROPA_DT_Cycle';
%noise seed
rng('default')

%% Loading .mat files
% previously defined disturbance profiles
%disturbances = load('Disturbances3'); 
%disturbances = load('DisturbancesLonger'); 
disturbances = load('disturbances_updown_EXP');
states = load('FinalPrimalLogAvg');

% previously computed system parameter profiles - used as "plant" true
% parameters
%parProfile = load('SmoothPlantParametersLonger');
parProfile = load('SmoothPlantParameters3'); 

% rig noise characteristics - compute previously from actual rig data
noise = load('RigNoise'); 

% Take out Noise
noise.input = 0.*noise.input;
noise.output = 0.*noise.output;

%% Simulation tuning
% Buffer length
BufferLength = 60; %[s]

% Optimization sampling time
nExec = 2; %[s]% Requested by Sigurd to make faster shadow prices update (EKF is not suggested to faster than 2 seconds)

% nExec = 10; %[s]% Initial

%plant parameters
parPlant = ParametersGasLiftModel;

%simulation parameters
nInit = 0; %[s]
%nFinal = 1*size(parProfile.thetaPlant,2); %[s] ! Adding a 60 second buffer in comparison to experimental duration (1244 ~ 22min) to emulate the measurement buffer of labview, which is "filled" before the experiment
nFinal = size(states.num,1)-1;
parPlant.T = 1; %rig measurements samplint time[s]
tgrid = (nInit:parPlant.T:nFinal)/60; %[min] one measurements per second

%initial condition
[dxPlant0,zPlant0,uPlant0,thetaPlant0] = InitialConditionGasLift(parPlant);

%states to measurement mapping function
parPlant.nMeas = 6;
parPlant.H = zeros(6,24);
parPlant.H(1,1) = 1e-2*60*1e3/parPlant.rho_o(1); %wro-oil rate from reservoir, well 1 [1e-2 kg/s] --> [L/min]
parPlant.H(2,2) = 1e-2*60*1e3/parPlant.rho_o(2); %wro-oil rate from reservoir, well 2
parPlant.H(3,3) = 1e-2*60*1e3/parPlant.rho_o(3); %wro-oil rate from reservoir, well 3
parPlant.H(4,7) = 1; %prh - riser head pressure well 1
parPlant.H(5,8) = 1; %prh - riser head pressure well 2
parPlant.H(6,9) = 1; %prh - riser head pressure well 3

% Model representing the rig
[F,~,~,~,~,~,~,~,~,~,~,~,~] = ErosionRigDynModel(parPlant);

% mockup controller dynamic parameters 
tau_C = 1;

%% Run configuration file
InitializationLabViewMain %here we use the same syntax as in the rig


%% Calculate Lagrange

len = size(states.num,1)-1;
shadow = [];

for i = 60:len
    I_vector = [states.num((i - BufferLength + 2):i + 1,9)';    % FI-101 [L/min]
                states.num((i - BufferLength + 2):i + 1,11)';   % FI-102 [L/min]
                states.num((i - BufferLength + 2):i + 1,13)';   % FI-103 [L/min]
                states.num((i - BufferLength + 2):i + 1,3)';    % FI-104 [sL/min]
                states.num((i - BufferLength + 2):i + 1,5)';    % FI-105 [sL/min]
                states.num((i - BufferLength + 2):i + 1,7)';    % FI-106 [sL/min]
                ones(3,BufferLength); %dummy values --> in the actual rig, they will the pressure at injection point (PI105, PI106, PI107). Not used here
                ones(3,BufferLength); %dummy values --> in the actual rig, they will the well temperature (TI101, TI102, TI103). Not used here
                ones(3,BufferLength); %dummy values --> in the actual rig, they will be dP in a given pipe section (dPI101, dPI102, dPI103). Not used here
                states.num((i - BufferLength + 2):i + 1,18)';    % PI-101 [mbar g]
                states.num((i - BufferLength + 2):i + 1,20)';    % PI-102 [mbar g] 
                states.num((i - BufferLength + 2):i + 1,22)';    % PI-103 [mbar g]
                states.num((i - BufferLength + 2):i + 1,24)'];   % PI-104 [bar g]
                
       
    % values of the input variables at the previus rig sampling time (dim = nu[7] X 1)
    P_vector = [(states.num(i,28)-0.004)./(0.02 - 0.004);% CV101 opening [-]
                (states.num(i,29)-0.004)./(0.02 - 0.004); % CV102 opening [-]
                (states.num(i,30)-0.004)./(0.02 - 0.004); % CV103 opening [-]
                1;%dummy values --> in the actual rig, they will the pump rotation. Not used here
                states.num(i,2); % FI-104sp [sL/min]
                states.num(i,4); % FI-105sp [sL/min]
                states.num(i,6)]; % FI-106sp [sL/min]
    % values of the inputs (gas lift) of the last optimization run (dim = nQg[3] X 1)
    O_vector = [states.num(i,3); states.num(i,5); states.num(i,7)];
    
    if rem(i,nExec) == 0
        LagrangeLabViewMain
    
        shadow = [shadow, Result];
    else
        shadow = [shadow, [0;0;0]];
    end
    
    disp(i);
end

%% plotting result
col = ['b' 'r' 'g' 'k' 'y' 'm']';

tgrid = (1:1:len)/60;

f1 = figure(1);
f1.Position = [400 50 750 250];
for i = 1:3
    plot(tgrid(61:2:end), movmean(shadow(i, 1:2:end-1),10), col(i), 'linewidth', 1.5)
    hold on
end
grid on

y_lab = '\lambda_i';
ylabel(['$',y_lab,'$'],'Interpreter','latex','FontSize',20)
xlabel('time [min]','FontSize',15)

xticks(0:5:(1/60)*(len - 1))
xlim([0 (1/60)*(len - 1)])

legend({'$\lambda_1$', '$\lambda_2$', '$\lambda_3$'},'Interpreter','latex','FontSize',15,'orientation','horizontal','location','northoutside')
